/***************************************************************************
 *   Copyright (C) 2013 by James Fuller,,,   *
 *   james@slakware12   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
//-----------------------------------------------------------------
#include "canvaspixmap.h"
#include <qbitmap.h>
#include <qpainter.h>
//-----------------------------------------------------------------
QWMatrix CanvasPixmap::wmFlipH = QWMatrix ( -1, 0, 0, 1, 0, 0 );
QWMatrix CanvasPixmap::wmFlipV = QWMatrix ( 1, 0, 0, -1, 0, 0 );
QWMatrix CanvasPixmap::wmOrtho [] = { QWMatrix ( 1, 0, 0, 1, 0, 0 ),
									  QWMatrix ( 0, -1, 1, 0, 0, 0 ),
									  QWMatrix ( -1, 0, 0, -1, 0, 0 ),
									  QWMatrix ( 0, 1, -1, 0, 0, 0 ) };
//-----------------------------------------------------------------
/**
* Creator
* @param canvas Where the object is represented
 */
CanvasPixmap::CanvasPixmap ( QCanvas * canvas )
			: QCanvasItem  (           canvas ) {
	pm = NULL;
	bFlippedX = false;
	bFlippedY = false;
	orthoAngle = none;
}
//-----------------------------------------------------------------
/**
 * Destructor
 */
CanvasPixmap::~CanvasPixmap()
{
	if ( pm ) delete pm;
}
//-----------------------------------------------------------------
/**
 * Determines if this object overlaps another QCanvasItem object
 * @param other The other object 
 * @return True if an overlap is detected
 */
bool CanvasPixmap::collidesWith ( const QCanvasItem * other ) const {
	if ( other == NULL ) return false;
	QRect rectOther = other->boundingRect ();	
	QRect rectThis  = boundingRect ();
	return rectOther.intersects ( rectThis );
}
//-----------------------------------------------------------------
/**
 * Determines if this object overlaps another QCanvasItem object
 * @param cs QCanvasSprite object to check for overlap
 * @param pi QCanvasPolygonalItem object to check for overlap
 * @param cr QCanvasRectangle object to check for overlap
 * @param ce QCanvasEllipse object to check for overlap
 * @param ct QCanvasText object to check for overlap
 * @return True if any specified object overlaps with this object
 */
bool CanvasPixmap::collidesWith( const QCanvasSprite* cs,
								 const QCanvasPolygonalItem* pi,
								 const QCanvasRectangle* cr,
								 const QCanvasEllipse* ce,
								 const QCanvasText* ct) const {
	if ( collidesWith ( cs ) ) return true;
	if ( collidesWith ( pi ) ) return true;
	if ( collidesWith ( cr ) ) return true;
	if ( collidesWith ( ce ) ) return true;
	if ( collidesWith ( ct ) ) return true;
	return false;
}
//-----------------------------------------------------------------
/**
 * Returns the bounding rectangle of the object
 * @return QRect
 */
QRect CanvasPixmap::boundingRect () const {
	int 	iWidth;
	int 	iHeight;
	QRect 	rect;
	QPoint	qp   ( x (), y () );

// Build a basic rectangle
	switch ( orthoAngle ) {
	case anti90:
	case anti180:
		iWidth  = iPixmapHeight;
		iHeight = iPixmapWidth;
		break;

	default:
		iWidth  = iPixmapWidth;
		iHeight = iPixmapHeight;
	}
	rect = QRect  ( 0, 0, iWidth, iHeight );
	
// translate for rotation (if any)
	switch ( orthoAngle ) {
	case anti90:
		qp -= QPoint ( 0, iPixmapWidth );
		break;

	case anti180:
		qp -= QPoint ( iPixmapWidth, iPixmapHeight );
		break;

	case anti270:
		qp -= QPoint ( iPixmapHeight, 0 );
		break;

	case none:
	default:
		break;
	}

// Translate for Flip (if any)
	if ( bFlippedX )
		qp -= QPoint ( iPixmapWidth, 0 );

	if ( bFlippedY )
		qp -= QPoint ( 0, iPixmapHeight );

	rect.moveBy ( qp.x (), qp.y () );
	return rect;
}
//-----------------------------------------------------------------
/**
 * Returns the width of the object
 * @param  none
 * @return Pixel count
 */
int CanvasPixmap::width ( void ) {
	int iWidth;

	switch ( orthoAngle ) {
	case anti90:
	case anti180:
		iWidth = iPixmapHeight;
		break;

	default:
		iWidth = iPixmapWidth;
	}
	return iWidth;
}
//-----------------------------------------------------------------
/**
 * Returns the height of the bitmap
 * @param  none
 * @return Pixel count
 */
int CanvasPixmap::height ( void ) {
	int iHeight;

	switch ( orthoAngle ) {
	case anti90:
	case anti180:
		iHeight = iPixmapWidth;
		break;

	default:
		iHeight = iPixmapHeight;
	}
	return iHeight;
}
//-----------------------------------------------------------------
/**
 * Returns a pointer to the internal QPixmap object 
 * @param  none
 * @return Pointer to QPixmap object
 */
QPixmap * CanvasPixmap::pixmap ( void ) {
	return pm;
}
//-----------------------------------------------------------------
/**
 * Set the QPixmap object that is to be displayed
 * @param _pm QPixmap object to display
 */
void CanvasPixmap::setPixmap ( QPixmap * _pm ) {
	iPixmapWidth  = 0;
	iPixmapHeight = 0;

	pm = _pm;
	if ( pm ) {
		iPixmapWidth  = pm->width ();
		iPixmapHeight = pm->height ();	
	} /* endif */
}
//-----------------------------------------------------------------
/**
 * Cause the pixmap to be flipped horizontally
 * @param  none
 */
void CanvasPixmap::flipH ( void ) {
	bFlippedX = ! bFlippedX;
	update ();
}
//-----------------------------------------------------------------
/**
 * Cause the pixmap to be flipped vertically
 * @param  
 */
void CanvasPixmap::flipV ( void ) {
	bFlippedY = ! bFlippedY;
	update ();
}
//-----------------------------------------------------------------
/**
 * Cause the pixmap to be rotated as previously specified
 * @param  none
 */
void CanvasPixmap::rotate ( void ) {
	int i = (int ) orthoAngle;
	i++;
	i = i % 4;
	setRotation ( (eRotate ( i ) ) );
}
//-----------------------------------------------------------------
/**
 * Return how far the pixmap is rotated (in 90 deg units)
 * @param  none
 * @return Number of 90 deg steps
 */
CanvasPixmap::eRotate CanvasPixmap::rotation ( void ) {
	return orthoAngle;
}
//-----------------------------------------------------------------
/**
 * Specifies the rotation of the pixmap (in 90 deg units)
 * @param _orthoAngle Number of 90 deg steps
 */
void CanvasPixmap::setRotation ( eRotate _orthoAngle ) {
	orthoAngle = _orthoAngle;
}
//-----------------------------------------------------------------
/**
 * Draw the object onto the canvas
 * @param painter 
 */
void CanvasPixmap::draw ( QPainter & painter ) {
	painter.save ();	
	painter.translate ( x (), y () ); // (base transform)

// save translated
	painter.save ();	

// optionally flip	(supplemental transform)
	if ( bFlippedX ) 
		painter.setWorldMatrix ( wmFlipH, true );

	if ( bFlippedY ) 
		painter.setWorldMatrix ( wmFlipV, true );
	
// set transform for rotation
	if ( orthoAngle ) 
		painter.setWorldMatrix ( wmOrtho [orthoAngle], true );
	
// Descendant class options (usually annotations, etc)
	drawTransformed ( painter );

	painter.drawPixmap ( 0, 0, *pm );
	painter.restore ();	

// de-translate
	drawUntransformed ( painter );
	painter.restore ();	
}
//-----------------------------------------------------------------
// Methods provided for overloading while still allowing the impure
//-----------------------------------------------------------------
// Draw onto rotated and/or flipped canvas
/**
 * Stub to perform additional drawing before rotation
 * @param painter 
 */
void CanvasPixmap::drawTransformed ( QPainter & painter ) {
}
//-----------------------------------------------------------------
// Draw text that needs to be orthogonal
/**
 * Stub to perform additional drawing after rotation
 * @param painter 
 */
void CanvasPixmap::drawUntransformed ( QPainter & painter ) {
}
//-----------------------------------------------------------------
/**
 * Unique representation of class derived from ScheditItem
 * @param  none
 * @return int
 */
int CanvasPixmap::rtti () const {
	return 1002;
}
//-----------------------------------------------------------------
