/***************************************************************************
 *   Copyright (C) 2013 by James Fuller,,,   *
 *   james@slakware12   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "colourfont.h"
//-------------------------------------------------------------------	
#include <qlayout.h>
//-------------------------------------------------------------------	
#include <kconfig.h>
//-------------------------------------------------------------------	
/**
 * Constructor for Font and Colour Dialog
 * @param parent Widget to become a part of
 * @param name Identifying name
 * @param li Address of LabelIni object that assists with settings
 */
ColourFont::ColourFont(QWidget    * parent, 
					   const char * name,
					   LabelIni   * li ) 
	: QWidget ( parent, name )
	, laPointSize ( "PointSize", NULL )
	, laWeight    ( "Weight",    NULL ) 
	, labelFont ( li )
	, fontDlg ( this, "Font" )
	, sp1 ( this )
	, labelColour ( "Colour", this )
	, dcb ( this, "Colour", &labelColour ) {
	QString sKey;
		
	setName ( name );
	QVBoxLayout * layout = new QVBoxLayout(this);
	layout->setAutoAdd(true);

// Bold section titles
	QFont font = labelColour.font ();
	font.setWeight (QFont::DemiBold );
	labelColour.setFont ( font );
	
	qsItalic = extendKey ( sKey, "Italic" );
	
// underline	
	qsUnderline = extendKey ( sKey, "Underline" );
	
// overline
	qsOverline = extendKey ( sKey, "Overline" );

// strikeout	
	qsStrikeout = extendKey ( sKey, "Strikeout" );

// Set PointSize and weight to decimal
	laPointSize.setBase ( 10 );
	laWeight.setBase    ( 10 );
	
	connect ( &dcb, SIGNAL (colorSelected(int)), 
			  this, SLOT ( render () ) );
}
//-------------------------------------------------------------------	
/**
 * Destructor
 */
ColourFont::~ColourFont () {
}
//-------------------------------------------------------------------
// Events
//-------------------------------------------------------------------
// Render is used to alter the colour of the displayed font
/**
 * Prvides the selected colour value to the Font dialog for display
 * @param  none
 */
void ColourFont::render (void) {
	fontDlg.setColor ( dcb.getColour () );
}
//-------------------------------------------------------------------
// Properties
//-------------------------------------------------------------------
/**
 * Save settings from the dialog 
 * @param config Ptr to KConfigGroup ( eg [Section] )
 * @param sKey Prefix for key names
 */
void ColourFont::write ( KConfigGroup * config, QString sKey ) {
	QStringList slRoot;
	int 		iIndex;
	QFont		font;
	QString     qs;

	slRoot = QStringList::split ( sKey, "." );
	font = fontDlg.font ();
	
// Family name (string)
	qs = extendKey ( sKey, "FamilyName" );
	config->writeEntry ( qs, font.family () );
	
// Point Size
	laPointSize.write ( config, 
						font.pointSize (),
						sKey );
	
// weight (boldness)	
	laWeight.write ( config,
					 font.weight (),
					 sKey );
	
// italic
	config->writeEntry ( qsItalic, font.italic () );
	
// underline	
	config->writeEntry ( qsUnderline, font.underline ()  );
	
// overline
	config->writeEntry ( qsOverline, font.overline ()  );

// strikeout	
	config->writeEntry ( qsStrikeout, font.strikeOut ()  );

// colour
	dcb.write ( config );
}
//-------------------------------------------------------------------
/**
 * Concatenates prefix and body of key name
 * @param sKey 
 * @param sName 
 * @return Amalgamated string
 */
QString ColourFont::extendKey ( QString sKey,
								QString sName ) {
	QStringList sl;
	QString 	qs;
	
	qs.sprintf ( "%s.%s", 
				 sKey.ascii (),
	 			 sName.ascii () );
	sl = QStringList::split ( '.', qs );
	return sl.join ( "." );
}
//-------------------------------------------------------------------
/**
 * Load settings into the dialog
 * @param config Ptr to KConfigGroup ( eg [Section] )
 * @param sKey Prefix for key names
*/
void ColourFont::read  ( KConfigGroup * config, QString sKey ) {
	QStringList slRoot;
	int 		iValue;
	bool		bValue;
	QString 	qs;
	QFont       font;
	QColor      qc;

	slRoot = QStringList::split ( sKey, "." );
	
	font = fontDlg.font ();			// get basic values first (from object)
	
// Family name (string)
	qs = extendKey ( sKey, "FamilyName" );
	qs = config->readEntry ( qs, font.family () );
	font.setFamily (qs);
			
// Point Size (protect against a zero value)
	iValue = laPointSize.read ( config, sKey );
	if ( iValue ) font.setPointSize ( iValue );
	
// weight (boldness)	
	iValue = laWeight.read ( config, sKey );
	font.setWeight ( iValue );
	
// italic
	bValue = config->readBoolEntry ( qsItalic, false );
	font.setItalic ( bValue );
	
// underline
	bValue = config->readBoolEntry ( qsUnderline, false );
	font.setUnderline ( bValue );
	
// overline
	bValue = config->readBoolEntry ( qsOverline, false );
	font.setOverline ( bValue );
	
// strikeout
	bValue = config->readBoolEntry ( qsStrikeout, false );
	font.setStrikeOut ( bValue );
	
// colour
	dcb.read ( config );		// sets internal index 'selected'
	
// Make it happen
	fontDlg.setColor ( dcb.getColour () );
	fontDlg.setFont ( font );
}
//-------------------------------------------------------------------
// get/set
//-------------------------------------------------------------------
/**
 * Returns a copy of the selected font
 * @param  none
 * @return QFont
 */
QFont ColourFont::getFont ( void ) {
	return fontDlg.font ();
}
//-------------------------------------------------------------------
// Drawing
//-------------------------------------------------------------------
/**
 * Copies specified parameters to the painting system
 * @param painter 
 */
void ColourFont::setupPainter ( QPainter& painter ) {
	QFont  font;
	QColor colour;
	QPen   pen;
	
// get data
	font   = getFont ();
	colour = dcb.getColour ();
	pen.setColor ( colour );
	
// setup painter
	painter.setFont ( font );
	painter.setPen  ( pen  );
}
//-------------------------------------------------------------------
#include "colourfont.moc"


